//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers 6
#property indicator_color1   clrDodgerBlue
#property indicator_color2   clrSandyBrown
#property indicator_color3   clrDodgerBlue
#property indicator_color4   clrSandyBrown
#property indicator_color5   clrSandyBrown
#property indicator_color6   clrHotPink
#property indicator_width3   2 
#property indicator_width4   2 
#property indicator_width5   2 
#property indicator_width6   2 
#property indicator_style1   STYLE_DOT
#property indicator_style2   STYLE_DOT

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame      = PERIOD_CURRENT;
extern int             DMILength       =    14;
extern bool            ShowADX         =  true;
extern bool            ShowADXR        = false;
extern int             Level           =    25;
extern int             Smooth          =     5;
extern ENUM_MA_METHOD  SmoothMethod    = MODE_EMA;
extern ENUM_APPLIED_PRICE PriceHigh    = PRICE_HIGH;
extern ENUM_APPLIED_PRICE PriceLow     = PRICE_LOW;
extern ENUM_APPLIED_PRICE PriceClose   = PRICE_CLOSE;
extern bool            alertsOn        = false;
extern bool            alertsOnCurrent = true;
extern bool            alertsMessage   = true;
extern bool            alertsSound     = false;
extern bool            alertsEmail     = false;
extern string          soundfile       = "alert2.wav";
extern bool            Interpolate     = true;

//
//
//
//
//

double DIp[];
double DIm[];
double ADX[];
double ADXda[];
double ADXdb[];
double ADXR[];
double averageDIp[];
double averageDIm[];
double averageTR[];
double trend[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(10);
   SetIndexBuffer(0,DIp);      SetIndexLabel(0,"DI+");
   SetIndexBuffer(1,DIm);      SetIndexLabel(1,"DI-");
   SetIndexBuffer(2,ADX);      SetIndexLabel(2,"ADX");
   SetIndexBuffer(3,ADXda);    SetIndexLabel(3,"ADX");
   SetIndexBuffer(4,ADXdb);    SetIndexLabel(4,"ADX");
   SetIndexBuffer(5,ADXR);     SetIndexLabel(5,"ADXR");
   SetIndexBuffer(6,averageDIp);
   SetIndexBuffer(7,averageDIm);
   SetIndexBuffer(8,averageTR);
   SetIndexBuffer(9,trend);
      for (int i=0;i<3;i++) SetIndexEmptyValue(i,0.00);

   //
   //
   //
   //
   //

      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame==-99);
      TimeFrame         = MathMax(TimeFrame,_Period);
      Smooth            = MathMax(Smooth,1);
         SetLevelValue(0,Level);

   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(TimeFrame)+" Wilder\'s DMI ("+DMILength+")");
   return(0);
}

int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-2);
           if (returnBars)  { DIp[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (TimeFrame == Period())
   {
      double sf = (DMILength-1.0)/DMILength;
      double arrc[3],arrp[2];
      if (trend[limit]==-1) CleanPoint(limit,ADXda,ADXdb);
      for (int i=limit; i>=0; i--)
      {
         arrc[0] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceHigh ,i  );
         arrc[1] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceLow  ,i  );
         arrc[2] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceClose,i+1);
         arrp[0] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceHigh ,i+1);
         arrp[1] = iMA(NULL,0,Smooth,0,SmoothMethod,PriceLow  ,i+1);
         ArraySort(arrc); ArraySort(arrp); 
         double currTR  = MathMax(arrc[2],arrc[1])-MathMin(arrc[0],arrc[1]);
         double DeltaHi = arrc[2] - arrp[1];
	      double DeltaLo = arrp[0] - arrc[0];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            averageDIp[i] = sf*averageDIp[i+1] + plusDM;
            averageDIm[i] = sf*averageDIm[i+1] + minusDM;
            averageTR[i]  = sf*averageTR[i+1]  + currTR;

         //
         //
         //
         //
         //
                  
            DIp[i] = 0.00;                   
            DIm[i] = 0.00;                   
            if (averageTR[i] > 0)
               {              
                  DIp[i] = 100.00 * averageDIp[i]/averageTR[i];
                  DIm[i] = 100.00 * averageDIm[i]/averageTR[i];
               }            

            if(ShowADX)
               {
                  double DX;
                  if((DIp[i] + DIm[i])>0) 
                       DX = 100*MathAbs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
                  else DX = 0.00;
                  ADX[i]   = sf*ADX[i+1] + DX/DMILength; 
                  ADXda[i] = EMPTY_VALUE;
                  ADXdb[i] = EMPTY_VALUE;
                  trend[i] = trend[i+1];
                     if (DIp[i]>DIm[i]) trend[i] =  1;
                     if (DIp[i]<DIm[i]) trend[i] = -1;
                     if (trend[i]==-1) PlotPoint(i,ADXda,ADXdb,ADX);
                     if(ShowADXR)
                         ADXR[i] = 0.5*(ADX[i] + ADX[i+DMILength]);
               }
      }   
      manageAlerts();
      return(0);      
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         DIp[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,PriceHigh,PriceLow,PriceClose,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,soundfile,0,y);
         DIm[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,PriceHigh,PriceLow,PriceClose,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,soundfile,1,y);
         ADXda[i] = EMPTY_VALUE;
         ADXdb[i] = EMPTY_VALUE;
            if (ShowADX)  ADX[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,PriceHigh,PriceLow,PriceClose,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,soundfile,2,y);
            if (ShowADX)  trend[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,PriceHigh,PriceLow,PriceClose,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,soundfile,9,y);
            if (ShowADXR) ADXR[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,PriceHigh,PriceLow,PriceClose,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,soundfile,5,y);

            //
            //
            //
            //
            //
      
            if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

            //
            //
            //
            //
            //

            datetime time = iTime(NULL,TimeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(int k = 1; k < n; k++)
               {
                  DIp[i+k] = DIp[i] + (DIp[i+n]-DIp[i])*k/n;
                  DIm[i+k] = DIm[i] + (DIm[i+n]-DIm[i])*k/n;
                     if (ShowADX)  ADX[i+k]  = ADX[i]  + (ADX[i+n] -ADX[i] )*k/n;
                     if (ShowADXR) ADXR[i+k] = ADXR[i] + (ADXR[i+n]-ADXR[i])*k/n;
               }
   }
   if (ShowADX) for (i=limit;i>=0;i--) if (trend[i]==-1) PlotPoint(i,ADXda,ADXdb,ADX);
   return(0);
}


//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      static string   type1 = "";
      static datetime time1 = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(type1,time1,"trend changed to up");
         if (trend[whichBar] ==-1) doAlert(type1,time1,"trend changed to down");
      }         
   }
}   

//
//
//
//
//

void doAlert(string& previousAlert, datetime& previousTime, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message =  timeFrameToString(_Period)+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" - Wilder's DMI "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" Wilder's DMI ",message);
          if (alertsSound)   PlaySound(soundfile);
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}